import Errors from './errors'

export default {
  mixins: [Errors],
  props: {
    model: {
      required: true
    },
    field: {
      type: Object,
      default () {
        return {}
      }
    },
    settings: Object,
    validations: Object,
    options: {
      type: Object,
      default () {
        return {}
      }
    },
    cssClass: {
      required: false
    },
    schema: Object,
    display: {
      type: Object,
      default () {
        return {}
      }
    },
    dispatch: String,
    commit: String,
    placeholder: String,
    disabled: {
      required: false
    },
    slots: Array,
    event: String,
    state: String,
    label: String,
    hint: String,
    name: {
      type: String,
      required: true
    },
    type: String,
    i18n: Array,
    v: {
      type: Object,
      default () {
        return {}
      }
    },
    default: {
      required: false
    },
    messages: Object,
    custom: {
      type: Object,
      default () {
        return {}
      }
    }
  },
  data () {
    return {
      fieldModel: this.model
    }
  },
  watch: {
    model (val) {
      this.fieldModel = val
      this.updateCheckbox()
      this.$emit('touch', this.name)
    },
    options (val, oldVal) {
      this.updateCheckbox()
    },
    $v (val) {
      this.$emit('touch', this.name)
    },
    fieldModel () {
      this.emit(this.type + ': ' + JSON.stringify(this.fieldModel))
    }
  },
  created () {
    this.fieldModel = this.model
  },
  computed: {
    invalid () {
      return this.$v.$invalid
    },
    disabledFunction () {
      if (this.disabled === true || typeof this.disabled === 'function') {
        return typeof this.disabled === 'function' ? this.disabled() : true
      } else {
        return false
      }
    },
    labelText () {
      return this.getText('labels', this.label || this.name)
    },
    placeholderText () {
      return this.placeholder ? this.getText('placeholders', this.placeholder) : ''
    },
    hintText () {
      return this.hint ? this.getText('hint', this.placeholder) : ''
    }
  },
  methods: {
    f (val) {
      return typeof val === 'function' ? val() : val
    },
    emit (who) {
      this.commitMutation()
      this.dispatchAction()
      this.emitEvent(this.event)
      if (this.name !== 'form' && this.type !== 'group') {
        this.emitEvent(this.settings.event)
      }
      this.$emit('input', this.fieldModel)
    },
    commitMutation () {
      if (this.commit && this.$store) {
        this.$store.commit(this.commit, this.fieldModel)
      }
    },
    dispatchAction () {
      if (this.dispatch && this.$store) {
        this.$store.dispatch(this.dispatch, {
          field: this.name,
          model: this.fieldModel
        })
      }
    },
    emitEvent (event) {
      if (event) {
        this.$emit('event', {
          name: event,
          field: this.name,
          model: this.fieldModel
        })
      }
    },
    updateCheckbox () {
      if (this.type === 'checkbox') {
        this.fieldModel = Object.keys(this.options).filter(key => this.fieldModel.indexOf(key) > -1)
      }
    }
  }
}
